@@ -49,7 +49,8 @@ on_about1_activate                     (GtkMenuItem     *menuitem,
   GtkWidget *about_dialog;
 	
   about_dialog = create_aboutdialog1();
-  gtk_window_set_icon(GTK_WINDOW(about_dialog), gtk_widget_render_icon(about_dialog, GTK_STOCK_ABOUT, GTK_ICON_SIZE_MENU, NULL));
+  gtk_window_set_icon (GTK_WINDOW(about_dialog), gtk_widget_render_icon (about_dialog, GTK_STOCK_ABOUT, GTK_ICON_SIZE_MENU, NULL));
+  gtk_window_set_transient_for (GTK_WINDOW(about_dialog), GTK_WINDOW(glb_win_app));
   gtk_widget_show (about_dialog);
 }
 
@@ -57,7 +58,7 @@ void
 on_app_safe_destroy                    (GtkObject       *object,
                                         gpointer         user_data)
 {
-  gtk_main_quit();
+    gtk_main_quit();
 }
 
 void
@@ -108,7 +109,11 @@ on_app_safe_delete_event               (GtkWidget       *widget,
                                         GdkEvent        *event,
                                         gpointer         user_data)
 {
-  fpm_quit();
+    if (glb_fpm_ini->enable_tray_icon)
+	fpm_tr_toggle_win_app(FALSE);
+    else
+	fpm_quit();
+	
   return TRUE;
 }
 
@@ -126,27 +131,21 @@ on_clist_main_button_press_event       (GtkWidget       *widget,
    * This is the press events.  We check to see how many clicks we get
    * within FPM_MAX_DOUBLE_CLICK_TIME seconds, currently set to 0.5.
    */
-  if(g_timer_elapsed(glb_timer_click, NULL) < FPM_MAX_DOUBLE_CLICK_TIME)
+  if((g_timer_elapsed(glb_timer_click, NULL) < FPM_MAX_DOUBLE_CLICK_TIME) && (glb_click_row == glb_cur_row))
   {
     glb_click_count++;
   }
   else
   {
     g_timer_reset(glb_timer_click);
-    glb_click_count=1;
+    glb_click_count = 1;
+    glb_click_row = glb_cur_row;
   }
  }
 
  if (event->button == 3) {
 
     glb_click_btn = TRUE;
-/*    GtkMenu *menu;
-    
-    menu = GTK_MENU(create_context_menu());
-    
-    gtk_menu_popup (menu, NULL, NULL, NULL, NULL,
-		    event->button, event->time);
-    return TRUE; */
  }
   return FALSE;
 }
@@ -164,7 +163,7 @@ on_clist_main_button_release_event     (GtkWidget       *widget,
    * more than once in the time allowed, consider it a double click.
    */
   if((g_timer_elapsed(glb_timer_click, NULL) < FPM_MAX_DOUBLE_CLICK_TIME) &&
-     (glb_click_count > 1) )
+     (glb_click_count > 1))
   {
     fpm_double_click(glb_edit_data);
   }
@@ -294,19 +293,10 @@ void
 on_togglebutton_showpass_toggled       (GtkToggleButton *togglebutton,
                                         gpointer         user_data)
 {
-  GtkEntry* entry_password;
+    GtkEntry* entry_password;
   
-  entry_password=GTK_ENTRY(lookup_widget (glb_win_edit, "entry_password"));
-  if (togglebutton->active)
-  {
-    gtk_entry_set_visibility (entry_password, TRUE);
-  }
-  else
-  {
-    gtk_entry_set_visibility (entry_password, FALSE);
-  }
-
-
+    entry_password = GTK_ENTRY(lookup_widget (glb_win_edit, "entry_password"));
+    gtk_entry_set_visibility(entry_password, togglebutton->active);
 }
 
 void
@@ -363,7 +353,7 @@ on_gpw_toggled                         (GtkEditable     *editable,
                                         gpointer         user_data)
 {
   GtkWidget* widget;
-  
+
   widget=lookup_widget(glb_win_misc, "entry_bits");
   if(strcmp("", gtk_entry_get_text(GTK_ENTRY(widget))))
   {
@@ -525,7 +515,7 @@ on_dialog_prefs_destroy_event          (GtkWidget       *widget,
                                         GdkEvent        *event,
                                         gpointer         user_data)
 {
-  
+
   return FALSE;
 }
 
@@ -727,7 +717,10 @@ void
 on_preferences2_activate               (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-    glb_win_misc = create_dialog_preferences();  
+    glb_win_misc = create_dialog_preferences();
+    gtk_window_set_transient_for (GTK_WINDOW(glb_win_misc), GTK_WINDOW(glb_win_app));
+//    gtk_window_set_destroy_with_parent (GTK_WINDOW(glb_win_misc), TRUE);
+
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "save_on_add")), glb_fpm_ini->save_on_add);
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "save_on_change")), glb_fpm_ini->save_on_change);
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "save_on_delete")), glb_fpm_ini->save_on_delete);
@@ -742,9 +735,25 @@ on_preferences2_activate               (GtkMenuItem     *menuitem,
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "search_match_case")), glb_fpm_ini->search_match_case);
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "search_limit_category")), glb_fpm_ini->search_limit_category);
 
-    if(glb_fpm_ini->create_backup)
-	gtk_widget_set_sensitive (GTK_WIDGET(lookup_widget (glb_win_misc, "hbox13")), TRUE);
+    if (glb_fpm_ini->create_backup)
+	gtk_widget_set_sensitive (GTK_WIDGET(lookup_widget (glb_win_misc, "backup_files_hbox")), TRUE);
 
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "enable_tray_icon")), glb_fpm_ini->enable_tray_icon);
+    if (glb_fpm_ini->enable_tray_icon) {
+	gtk_widget_set_sensitive (GTK_WIDGET(lookup_widget (glb_win_misc, "tr_always_visible")), TRUE);
+	gtk_widget_set_sensitive (GTK_WIDGET(lookup_widget (glb_win_misc, "tr_auto_hide")), TRUE);
+	gtk_widget_set_sensitive (GTK_WIDGET(lookup_widget (glb_win_misc, "tr_auto_lock")), TRUE);
+}
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "tr_always_visible")), glb_fpm_ini->tr_always_visible);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "tr_auto_hide")), glb_fpm_ini->tr_auto_hide);
+    gtk_spin_button_set_value (GTK_SPIN_BUTTON(lookup_widget (glb_win_misc, "tr_auto_hide_minutes")), glb_fpm_ini->tr_auto_hide_minutes);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "tr_auto_lock")), glb_fpm_ini->tr_auto_lock);
+    gtk_spin_button_set_value (GTK_SPIN_BUTTON(lookup_widget (glb_win_misc, "tr_auto_lock_minutes")), glb_fpm_ini->tr_auto_lock_minutes);
+    if (!glb_fpm_ini->enable_tray_icon) {
+	gtk_widget_set_sensitive (GTK_WIDGET(lookup_widget (glb_win_misc , "tr_auto_hide_align")), FALSE);
+	gtk_widget_set_sensitive (GTK_WIDGET(lookup_widget (glb_win_misc , "tr_auto_lock_align")), FALSE);
+    }
+    
     if (gtk_dialog_run (GTK_DIALOG (glb_win_misc)) == GTK_RESPONSE_OK)
     {
 	glb_fpm_ini->save_on_add = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "save_on_add")));
@@ -753,15 +762,31 @@ on_preferences2_activate               (GtkMenuItem     *menuitem,
 	glb_fpm_ini->save_on_quit = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "save_on_quit")));
 	glb_fpm_ini->create_backup = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "create_backup")));
 	glb_fpm_ini->number_backup_files = gtk_spin_button_get_value (GTK_SPIN_BUTTON(lookup_widget (glb_win_misc, "number_backup_files")));
-	
+
 	glb_fpm_ini->search_in_title = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "search_in_title")));
 	glb_fpm_ini->search_in_url = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "search_in_url")));
 	glb_fpm_ini->search_in_user = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "search_in_user")));
 	glb_fpm_ini->search_in_notes = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "search_in_notes")));
 	glb_fpm_ini->search_match_case = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "search_match_case")));
 	glb_fpm_ini->search_limit_category = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "search_limit_category")));
-	
-	fpm_ini_save();	
+
+	glb_fpm_ini->enable_tray_icon = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "enable_tray_icon")));
+	glb_fpm_ini->tr_always_visible = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "tr_always_visible")));
+	glb_fpm_ini->tr_auto_hide = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "tr_auto_hide")));
+	glb_fpm_ini->tr_auto_hide_minutes = gtk_spin_button_get_value (GTK_SPIN_BUTTON(lookup_widget (glb_win_misc, "tr_auto_hide_minutes")));
+	glb_fpm_ini->tr_auto_lock = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "tr_auto_lock")));
+	glb_fpm_ini->tr_auto_lock_minutes = gtk_spin_button_get_value (GTK_SPIN_BUTTON(lookup_widget (glb_win_misc, "tr_auto_lock_minutes")));
+
+	if (glb_fpm_ini->enable_tray_icon)
+	    fpm_tray_icon();
+	else {
+	    fpm_tr_cleanup();
+	    gtk_widget_hide (GTK_WIDGET(lookup_widget (glb_win_app , "hide_to_tray")));
+//	    g_signal_handlers_disconnect_by_func (glb_win_app, G_CALLBACK (on_app_safe_event), NULL);
+	    g_source_remove_by_user_data (glb_win_app);
+	}
+
+	fpm_ini_save();
     }
     gtk_widget_destroy (glb_win_misc);
 }
@@ -770,10 +795,7 @@ void
 on_create_backup_toggled               (GtkToggleButton *togglebutton,
                                         gpointer         user_data)
 {
-    if (gtk_toggle_button_get_active(togglebutton))
-	gtk_widget_set_sensitive (GTK_WIDGET(lookup_widget (glb_win_misc , "hbox13")), TRUE);
-    else
-	gtk_widget_set_sensitive (GTK_WIDGET(lookup_widget (glb_win_misc , "hbox13")), FALSE);
+    gtk_widget_set_sensitive (GTK_WIDGET(lookup_widget (glb_win_misc , "backup_files_hbox")), togglebutton->active);
 }
 
 void
@@ -821,7 +843,7 @@ on_find_activate                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget (glb_win_app, "find_match_case")), glb_fpm_ini->search_match_case);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget (glb_win_app, "find_match_case")), glb_fpm_ini->search_match_case);
 
     gtk_widget_show (GTK_WIDGET(lookup_widget (glb_win_app, "findbar")));
     gtk_widget_grab_focus (GTK_WIDGET(lookup_widget (glb_win_app, "find_entry")));
@@ -855,7 +877,7 @@ void
 on_find_entry_changed                  (GtkEditable     *editable,
                                         gpointer         user_data)
 {
-    fpm_search(gtk_editable_get_chars(editable, 0, -1));
+    fpm_search(gtk_editable_get_chars(editable, 0, -1), FALSE);
 }
 
 
@@ -863,16 +885,7 @@ void
 on_find_match_case_toggled             (GtkToggleButton *togglebutton,
                                         gpointer         user_data)
 {
-
-    if (togglebutton->active)
-    {
-	glb_fpm_ini->search_match_case = TRUE;
-    }
-    else
-    {
-	glb_fpm_ini->search_match_case = FALSE;
-    }
-  
+    glb_fpm_ini->search_match_case = togglebutton->active;
     gtk_widget_activate(GTK_WIDGET(lookup_widget (glb_win_app, "find_entry")));
 }
 
@@ -881,6 +894,141 @@ void
 on_find_entry_activate                 (GtkEntry        *entry,
                                         gpointer         user_data)
 {
-    fpm_search(gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1));
+    fpm_search(gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1), TRUE);
 }
 
+void
+tray_icon_on_click			(GtkStatusIcon *status_icon, 
+					 gpointer user_data)
+{
+    if (glb_locked) {
+	gtk_status_icon_set_visible(glb_tray_icon, FALSE);
+	glb_locked = FALSE;
+	unlock_fpm_file();
+	fpm_init(glb_filename);
+    } else {
+	fpm_tr_toggle_win_app(FALSE);
+    }
+}
+
+void
+tray_icon_on_menu			(GtkStatusIcon *status_icon, guint button, 
+                    			 guint activate_time, gpointer user_data)
+{
+    GtkMenu *menu;
+
+    menu = GTK_MENU(create_tray_menu());
+
+    if (!glb_locked) {
+	gtk_widget_hide (GTK_WIDGET(lookup_widget (GTK_WIDGET(menu) , "unlock1")));
+	gtk_widget_show (GTK_WIDGET(lookup_widget (GTK_WIDGET(menu) , "lock1")));
+	gtk_widget_show (GTK_WIDGET(lookup_widget (GTK_WIDGET(menu) , "show1")));
+	gtk_widget_show (GTK_WIDGET(lookup_widget (GTK_WIDGET(menu) , "tr_about1")));
+    } else {
+	gtk_widget_show (GTK_WIDGET(lookup_widget (GTK_WIDGET(menu) , "unlock1")));
+	gtk_widget_hide (GTK_WIDGET(lookup_widget (GTK_WIDGET(menu) , "show1")));
+	gtk_widget_hide (GTK_WIDGET(lookup_widget (GTK_WIDGET(menu) , "lock1")));
+	gtk_widget_hide (GTK_WIDGET(lookup_widget (GTK_WIDGET(menu) , "tr_about1")));
+    }
+
+    gtk_menu_popup (menu, NULL, NULL, gtk_status_icon_position_menu, status_icon,
+    0, gtk_get_current_event_time());
+}
+
+void
+on_enable_tray_icon_toggled            (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+    gtk_widget_set_sensitive (GTK_WIDGET(lookup_widget (glb_win_misc , "tr_always_visible")), togglebutton->active);
+    gtk_widget_set_sensitive (GTK_WIDGET(lookup_widget (glb_win_misc , "tr_auto_hide")), togglebutton->active);
+    gtk_widget_set_sensitive (GTK_WIDGET(lookup_widget (glb_win_misc , "tr_auto_lock")), togglebutton->active);
+
+    gtk_widget_set_sensitive (GTK_WIDGET(lookup_widget (glb_win_misc , "tr_auto_hide_align")), togglebutton->active & glb_fpm_ini->tr_auto_hide);
+    gtk_widget_set_sensitive (GTK_WIDGET(lookup_widget (glb_win_misc , "tr_auto_lock_align")), togglebutton->active & glb_fpm_ini->tr_auto_lock);
+}
+
+void
+on_lock1_activate                      (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+    fpm_lock();
+}
+
+
+void
+on_tr_auto_hide_toggled                (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+    gtk_widget_set_sensitive (GTK_WIDGET(lookup_widget (glb_win_misc , "tr_auto_hide_align")), togglebutton->active);
+    glb_fpm_ini->tr_auto_hide = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "tr_auto_hide")));
+}
+
+
+void
+on_tr_auto_lock_toggled                (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+    gtk_widget_set_sensitive (GTK_WIDGET(lookup_widget (glb_win_misc , "tr_auto_lock_align")), togglebutton->active);
+    glb_fpm_ini->tr_auto_lock = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(lookup_widget (glb_win_misc, "tr_auto_lock")));
+}
+
+void
+on_hide_to_tray_activate               (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+    fpm_tr_toggle_win_app(FALSE);
+}
+
+
+gboolean
+on_app_safe_window_state_event         (GtkWidget       *widget,
+                                        GdkEventWindowState	 *event,
+                                        gpointer         user_data)
+{
+
+    if (event->changed_mask & GDK_WINDOW_STATE_ICONIFIED)
+	glb_minimized = (event->new_window_state & GDK_WINDOW_STATE_ICONIFIED) ? 1 : 0;
+
+    return FALSE;
+}
+
+void
+on_show1_activate                      (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+    fpm_tr_toggle_win_app(FALSE);
+}
+
+gboolean
+on_app_safe_focus_in_event             (GtkWidget       *widget,
+                                        GdkEventFocus   *event,
+                                        gpointer         user_data)
+{
+    if (glb_fpm_ini->tr_auto_hide)
+	g_source_remove_by_user_data (glb_win_app);
+
+    return FALSE;
+}
+
+
+gboolean
+on_app_safe_focus_out_event            (GtkWidget       *widget,
+                                        GdkEventFocus   *event,
+                                        gpointer         user_data)
+{
+    if (!fpm_window_check() && glb_fpm_ini->tr_auto_hide && !glb_minimized)
+	g_timeout_add_seconds(glb_fpm_ini->tr_auto_hide_minutes*60, (GSourceFunc) fpm_auto_hide, glb_win_app);
+
+  return FALSE;
+}
+
+
+void
+on_unlock1_activate                    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+    gtk_status_icon_set_visible(glb_tray_icon, FALSE);
+    glb_locked = FALSE;
+    unlock_fpm_file();
+    fpm_init(glb_filename);
+}
