@@ -31,13 +31,17 @@
 #include <sys/file.h>
 #include <sys/stat.h>
 #include <errno.h>
-#include <string.h> 
+#include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
 
+#include <gdk/gdk.h>
+#include <gdk/gdkx.h>
+#include <X11/Xatom.h>
+
 /* GLOBAL VARIABLES */
 GtkWidget* glb_win_app;   /* The main window for the application   */
-GtkTreeView* glb_clist_main; /* Password CList on main window         */
+GtkTreeView* glb_clist_main; /* Password CList on main window      */
 
 GtkWidget* glb_win_misc;  /* Misc window, used for dialogs         */
 
@@ -53,6 +57,17 @@ gint glb_click_count;	  /* Click count used for double clicks    */
 gchar* new_salt;	  /* Salt to use on next save.             */
 gchar* old_salt;	  /* Salt to used on last save.            */
 gboolean glb_click_btn;	  /* Right button clicked -> context menu  */
+gint glb_click_row;       /* Store row for first click on dblclick */
+gboolean glb_minimized;	  /* Is program minimized?		   */
+gboolean glb_locked;	  /* Is program locked?			   */
+gboolean glb_timeout; /* Running idle check?		   */
+gboolean glb_lock_timeout; /* Running lock check?		   */
+gint glb_idle_min;
+
+gboolean glb_events;
+gboolean glb_test;
+
+GtkStatusIcon *glb_tray_icon; /* Tray icon */
 
 fpm_ini* glb_fpm_ini;	  /* Store FPM settings. */
 
@@ -236,7 +251,12 @@ fpm_new_passitem(GtkWidget** win_edit_ptr, fpm_data** data_ptr)
     new_data->arg=g_strdup("");
     new_data->user=g_strdup("");
     new_data->notes=g_strdup("");
-    new_data->category=g_strdup("");
+
+    if (g_utf8_collate(glb_last_category, FPM_ALL_CAT_MSG) == 0)
+	new_data->category=g_strdup("");
+    else
+	new_data->category=g_strdup(glb_last_category);
+
     new_data->launcher=g_strdup("");
     *data_ptr = new_data;
 
@@ -266,7 +286,6 @@ fpm_edit_passitem(GtkWidget** win_edit_ptr, fpm_data* data)
   g_assert(glb_cat_string_list!=NULL);
   gtk_combo_set_popdown_strings(combo, glb_cat_string_list);
 
-  
   fpm_create_launcher_string_list();
   g_assert(glb_launcher_string_list!=NULL);
   combo = GTK_COMBO(lookup_widget(*win_edit_ptr, "combo_edit_launcher"));
@@ -292,7 +311,8 @@ fpm_edit_passitem(GtkWidget** win_edit_ptr, fpm_data* data)
 
   buffer = gtk_text_view_get_buffer(notes);
   gtk_text_buffer_insert_at_cursor(buffer, data->notes ,strlen(data->notes));
-  
+
+  gtk_window_set_transient_for (GTK_WINDOW(*win_edit_ptr), GTK_WINDOW(glb_win_app));
   gtk_widget_show(*win_edit_ptr);
   gtk_widget_set_sensitive(glb_win_app, FALSE);
 
@@ -432,11 +452,13 @@ fpm_init(char* opt_file_name)
 
   glb_launcher_string_list=NULL;
 
+  fpm_ini_load();
+
   if( glb_win_app == NULL)
   {
 	/* First run */
-	fpm_ini_load();
 	glb_win_app = create_app_safe();
+	gtk_widget_hide (GTK_WIDGET(lookup_widget (glb_win_app , "hide_to_tray")));	
   }
   else
 	gtk_widget_hide(glb_win_app);
@@ -492,8 +514,7 @@ void
 fpm_quit(void)
 {
   glb_quitting=TRUE;
-  if (glb_dirty)
-  {
+  if (glb_dirty) {
     if (glb_fpm_ini->save_on_quit)
 	passfile_save(glb_filename);		
     else
@@ -501,6 +522,9 @@ fpm_quit(void)
 		 G_CALLBACK(fpm_dialog_answer_save));
   }
 
+  if (glb_fpm_ini->enable_tray_icon)
+    glb_tray_icon = (g_object_unref (glb_tray_icon), NULL);
+
   fpm_ini_save ();
 
   gtk_main_quit();
@@ -625,10 +649,9 @@ fpm_jump(fpm_data* data)
 
     if ( strlen(launcher->cmdline) > 0 )
     {
-	    cmd=fpm_create_cmd_line(launcher->cmdline, data->arg,
+        cmd = fpm_create_cmd_line(launcher->cmdline, data->arg,
 		 data->user, data->password);
-
-	    fpm_execute_shell(cmd);
+	fpm_execute_shell(cmd);
     }
     else
     {
@@ -685,10 +708,20 @@ fpm_check_password()
   fpm_clist_populate_cat_list();
 
   gtk_window_resize (GTK_WINDOW(glb_win_app), glb_fpm_ini->main_width, glb_fpm_ini->main_height);
+  if ((glb_fpm_ini->main_x != -1) && (glb_fpm_ini->main_y != -1))
+    gtk_window_move(GTK_WINDOW(glb_win_app), glb_fpm_ini->main_x, glb_fpm_ini->main_y);
+
+  if (glb_fpm_ini->enable_tray_icon)
+    fpm_tray_icon();
+
+  gtk_window_set_skip_taskbar_hint(GTK_WINDOW(glb_win_app), FALSE);
+  gtk_window_set_skip_pager_hint(GTK_WINDOW(glb_win_app), FALSE);
+  gtk_window_present(GTK_WINDOW(glb_win_app));
+  gtk_window_deiconify(GTK_WINDOW(glb_win_app));
 
-  gtk_widget_show(glb_win_app);
   gtk_widget_destroy(glb_win_misc);
 
+  glb_minimized = FALSE;
 }
 
 
@@ -813,8 +846,6 @@ void fpm_create_launcher_string_list()
   }
 }
 
-
-
 gchar* fpm_create_cmd_line(gchar* cmd, gchar* arg, gchar* user, gchar* pass)
 {
   gchar* result;
@@ -885,7 +916,7 @@ void fpm_question(GtkWindow* win, gchar* message, GCallback callback)
             "response", 
 	     G_CALLBACK (callback),
     	     NULL);
-    	     
+
     gint result = gtk_dialog_run (GTK_DIALOG (dw));
     if(result != GTK_RESPONSE_NONE) gtk_widget_destroy (dw);
 }
@@ -897,7 +928,7 @@ void fpm_ini_load()
     gsize count;
     gint i;
     gint *columns;
-    
+
     glb_fpm_ini = g_malloc0(sizeof(fpm_ini));
 
     gchar *fpm_ini_file = g_build_filename (g_get_home_dir(), ".fpm", "fpm.ini", NULL);
@@ -906,22 +937,32 @@ void fpm_ini_load()
     if (!g_key_file_load_from_file (keyfile, fpm_ini_file, G_KEY_FILE_NONE, NULL))
     {
 	/* No ini file found -> set defaults and save */
-        glb_fpm_ini->save_on_add = TRUE;
-        glb_fpm_ini->save_on_change = TRUE;
-        glb_fpm_ini->save_on_delete = FALSE;
-        glb_fpm_ini->save_on_quit = FALSE;
-        glb_fpm_ini->create_backup = TRUE;
-        glb_fpm_ini->number_backup_files = 5;
-	glb_fpm_ini->main_width = 500;	
+	glb_fpm_ini->save_on_add = TRUE;
+	glb_fpm_ini->save_on_change = TRUE;
+	glb_fpm_ini->save_on_delete = FALSE;
+	glb_fpm_ini->save_on_quit = FALSE;
+	glb_fpm_ini->create_backup = TRUE;
+	glb_fpm_ini->number_backup_files = 5;
+	glb_fpm_ini->main_x = -1;
+	glb_fpm_ini->main_y = -1;
+	glb_fpm_ini->main_width = 500;
 	glb_fpm_ini->main_height = 350;
 	glb_fpm_ini->last_category = FPM_ALL_CAT_MSG;
-	glb_last_category = FPM_ALL_CAT_MSG;
 	glb_fpm_ini->search_in_title = TRUE;
 	glb_fpm_ini->search_in_url = FALSE;
 	glb_fpm_ini->search_in_user = FALSE;
 	glb_fpm_ini->search_in_notes = FALSE;
 	glb_fpm_ini->search_match_case = FALSE;
 	glb_fpm_ini->search_limit_category = TRUE;
+	glb_fpm_ini->enable_tray_icon = FALSE;
+	glb_fpm_ini->tr_always_visible = FALSE;
+	glb_fpm_ini->tr_auto_hide = FALSE;
+	glb_fpm_ini->tr_auto_hide_minutes = 15;
+	glb_fpm_ini->tr_auto_lock = FALSE;
+	glb_fpm_ini->tr_auto_lock_minutes = 15;
+
+	glb_last_category = FPM_ALL_CAT_MSG;
+
 	fpm_ini_save();
     } else {
 	glb_fpm_ini->save_on_add = g_key_file_get_boolean (keyfile, "general", "save_on_add", NULL);
@@ -934,11 +975,20 @@ void fpm_ini_load()
         glb_fpm_ini->last_category = g_key_file_get_string (keyfile, "general", "last_category", &error);
 	if (error != NULL)
 	    glb_fpm_ini->last_category = FPM_ALL_CAT_MSG;
-	    
+
+	error = NULL;
+	glb_fpm_ini->main_x = g_key_file_get_integer (keyfile, "general", "main_x", &error);
+	if (error != NULL)
+	    glb_fpm_ini->main_x = -1;
+	error = NULL;
+	glb_fpm_ini->main_y = g_key_file_get_integer (keyfile, "general", "main_y", &error);
+	if (error != NULL)
+	    glb_fpm_ini->main_y = -1;
+
 	error = NULL;
 	glb_fpm_ini->main_width = g_key_file_get_integer (keyfile, "general", "main_width", &error);
 	if (error != NULL)
-	    glb_fpm_ini->main_width = 500;	
+	    glb_fpm_ini->main_width = 500;
 
 	error = NULL;
 	glb_fpm_ini->main_height = g_key_file_get_integer (keyfile, "general", "main_height", &error);
@@ -953,7 +1003,7 @@ void fpm_ini_load()
 	} else {
 	    memcpy(glb_fpm_ini->columns_width, columns, sizeof(glb_fpm_ini->columns_width));
 	    g_free(columns);
-	}    
+	}
 
 	error = NULL;
 	glb_fpm_ini->search_in_title = g_key_file_get_boolean (keyfile, "search", "search_in_title", &error);
@@ -971,6 +1021,27 @@ void fpm_ini_load()
 	    glb_fpm_ini->search_limit_category = TRUE;
 
 	glb_last_category = glb_fpm_ini->last_category;
+
+	error = NULL;
+	glb_fpm_ini->enable_tray_icon = g_key_file_get_boolean (keyfile, "tray_icon", "enable_tray_icon", &error);
+	if (error != NULL)
+	    glb_fpm_ini->enable_tray_icon = FALSE;
+
+	glb_fpm_ini->tr_always_visible = g_key_file_get_boolean (keyfile, "tray_icon", "tr_always_visible", NULL);
+	glb_fpm_ini->tr_auto_hide = g_key_file_get_boolean (keyfile, "tray_icon", "tr_auto_hide", NULL);
+
+	error = NULL;
+	glb_fpm_ini->tr_auto_hide_minutes = g_key_file_get_integer (keyfile, "tray_icon", "tr_auto_hide_minutes", &error);
+	if (error != NULL)
+	    glb_fpm_ini->tr_auto_hide_minutes = 15;
+
+	glb_fpm_ini->tr_auto_lock = g_key_file_get_boolean (keyfile, "tray_icon", "tr_auto_lock", NULL);
+
+	error = NULL;
+	glb_fpm_ini->tr_auto_lock_minutes = g_key_file_get_integer (keyfile, "tray_icon", "tr_auto_lock_minutes", &error);
+	if (error != NULL)
+	    glb_fpm_ini->tr_auto_lock_minutes = 15;
+
     }
 
     g_key_file_free (keyfile);
@@ -980,17 +1051,20 @@ void fpm_ini_save()
 {
     GKeyFile *keyfile;
     GtkTreeViewColumn *column;
-    gint width, height;
+    gint x, y, width, height;
     gint i;
-    
+
     if (glb_win_app == NULL) {
+	x = -1;
+	y = -1;
 	width = 500;
 	height = 350;
 	for (i=0; i< FPM_NUM_COL; i++) 
 	    glb_fpm_ini->columns_width[i] = 150;
     } else {
 	gtk_window_get_size (GTK_WINDOW(glb_win_app), &width, &height);
-	    
+	gtk_window_get_position(GTK_WINDOW(glb_win_app), &x, &y);
+
 	for (i=0; i< FPM_NUM_COL; i++) {
 	    column = gtk_tree_view_get_column (glb_clist_main, i);
 	    glb_fpm_ini->columns_width[i] = gtk_tree_view_column_get_width (column);
@@ -998,7 +1072,7 @@ void fpm_ini_save()
     }
 
     gchar *fpm_ini_file = g_build_filename (g_get_home_dir(), ".fpm", "fpm.ini", NULL);
-    
+
     keyfile = g_key_file_new ();
 
     g_key_file_set_boolean (keyfile, "general", "save_on_add", glb_fpm_ini->save_on_add);
@@ -1008,6 +1082,8 @@ void fpm_ini_save()
     g_key_file_set_boolean (keyfile, "general", "create_backup", glb_fpm_ini->create_backup);
     g_key_file_set_integer (keyfile, "general", "number_backup_files", glb_fpm_ini->number_backup_files);
     g_key_file_set_string (keyfile, "general", "last_category", glb_last_category);
+    g_key_file_set_integer (keyfile, "general", "main_x", x);
+    g_key_file_set_integer (keyfile, "general", "main_y", y);
     g_key_file_set_integer (keyfile, "general", "main_width", width);
     g_key_file_set_integer (keyfile, "general", "main_height", height);
     g_key_file_set_integer_list (keyfile, "general", "columns_width", glb_fpm_ini->columns_width, 3);
@@ -1019,6 +1095,13 @@ void fpm_ini_save()
     g_key_file_set_boolean (keyfile, "search", "search_match_case", glb_fpm_ini->search_match_case);
     g_key_file_set_boolean (keyfile, "search", "search_limit_category", glb_fpm_ini->search_limit_category);
 
+    g_key_file_set_boolean (keyfile, "tray_icon", "enable_tray_icon", glb_fpm_ini->enable_tray_icon);
+    g_key_file_set_boolean (keyfile, "tray_icon", "tr_always_visible", glb_fpm_ini->tr_always_visible);
+    g_key_file_set_boolean (keyfile, "tray_icon", "tr_auto_hide", glb_fpm_ini->tr_auto_hide);
+    g_key_file_set_integer (keyfile, "tray_icon", "tr_auto_hide_minutes", glb_fpm_ini->tr_auto_hide_minutes);
+    g_key_file_set_boolean (keyfile, "tray_icon", "tr_auto_lock", glb_fpm_ini->tr_auto_lock);
+    g_key_file_set_integer (keyfile, "tray_icon", "tr_auto_lock_minutes", glb_fpm_ini->tr_auto_lock_minutes);
+
     gchar *save_data = g_key_file_to_data (keyfile, NULL, NULL);
 
     g_file_set_contents (fpm_ini_file, save_data, -1, NULL);
@@ -1026,7 +1109,7 @@ void fpm_ini_save()
     g_key_file_free (keyfile);
 }
 
-void fpm_search(gchar *search_text)
+void fpm_search(gchar *search_text, gboolean select_first)
 {
     fpm_data *data;
     GList *list;
@@ -1047,14 +1130,14 @@ void fpm_search(gchar *search_text)
         title = data->title;
         url = data->arg;
 	user = data->user;
-    	notes = data->notes;
+	notes = data->notes;
 
         if ((!strcmp(glb_last_category, data->category) && glb_fpm_ini->search_limit_category)
 	    || ((!strcmp(glb_last_category, FPM_ALL_CAT_MSG)
 	    || !glb_fpm_ini->search_limit_category)))
 	{
 
-    	    if (!glb_fpm_ini->search_match_case) {
+	    if (!glb_fpm_ini->search_match_case) {
 		title = g_utf8_casefold(title, -1);
 		url = g_utf8_casefold(url, -1);
 		user = g_utf8_casefold(user, -1);
@@ -1077,15 +1160,18 @@ void fpm_search(gchar *search_text)
 	    	    -1);
 	    }
 	}
-	list=g_list_next(list);
+	list = g_list_next(list);
     }
+    
+    if (select_first)
+	gtk_widget_grab_focus(GTK_WIDGET(glb_clist_main));
 }
 
 void fpm_execute_shell(gchar *cmd)
 {
     gchar *execute;
     int ret;
-    
+
     execute = g_strjoin(NULL, cmd, " &", NULL);
     ret = system(execute);
 }
@@ -1093,8 +1179,140 @@ void fpm_execute_shell(gchar *cmd)
 void fpm_statusbar_push(gchar *message)
 {
     GtkStatusbar *statusbar;
-    
+
     statusbar = GTK_STATUSBAR(lookup_widget (glb_win_app, "statusbar"));
     gtk_statusbar_pop(statusbar, 1);
     gtk_statusbar_push(statusbar, 1, message);
 }
+
+gboolean fpm_auto_hide() {
+
+    fpm_tr_toggle_win_app(TRUE);
+
+    return FALSE;
+}
+
+void fpm_tray_icon() {
+
+    if (glb_tray_icon == NULL) {
+	glb_tray_icon = gtk_status_icon_new();
+
+	gtk_status_icon_set_visible(glb_tray_icon, FALSE);
+	g_signal_connect(G_OBJECT(glb_tray_icon), "activate", 
+			G_CALLBACK(tray_icon_on_click), NULL);
+	g_signal_connect(G_OBJECT(glb_tray_icon), "popup-menu",
+			G_CALLBACK(tray_icon_on_menu), NULL);
+	gtk_status_icon_set_from_file (glb_tray_icon, PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps/logo.png");
+    }
+
+    gtk_status_icon_set_tooltip(glb_tray_icon, _("Figaro's Password Manager 2"));
+    gtk_status_icon_set_visible(glb_tray_icon, glb_fpm_ini->tr_always_visible);
+    gtk_widget_show (GTK_WIDGET(lookup_widget (glb_win_app , "hide_to_tray")));
+}
+
+gboolean fpm_hide_win_app() {
+    gtk_widget_hide(glb_win_app);
+    return FALSE;
+}
+
+void fpm_tr_toggle_win_app(gboolean force_hide) {
+
+    static int x, y;
+    static GdkScreen *screen;
+
+    GdkRectangle  bounds;
+    gulong        data[4];
+    Display      *dpy;
+    GdkWindow    *gdk_window;
+
+
+    if (glb_minimized && !force_hide) {
+	gtk_window_set_screen(GTK_WINDOW(glb_win_app), screen);
+	gtk_window_move(GTK_WINDOW(glb_win_app), x, y);
+
+	gtk_widget_show(glb_win_app);
+	gtk_window_deiconify(GTK_WINDOW(glb_win_app));
+	gtk_window_present(GTK_WINDOW(glb_win_app));
+
+	gtk_window_set_skip_taskbar_hint(GTK_WINDOW(glb_win_app), FALSE);
+	gtk_window_set_skip_pager_hint(GTK_WINDOW(glb_win_app), FALSE);
+
+	if (!glb_fpm_ini->tr_always_visible)
+	    gtk_status_icon_set_visible(glb_tray_icon, FALSE);
+
+	glb_minimized = FALSE;
+
+    } else {
+
+	gtk_status_icon_set_visible(glb_tray_icon, TRUE);
+	gtk_window_set_skip_taskbar_hint(GTK_WINDOW(glb_win_app), TRUE);
+	gtk_window_set_skip_pager_hint(GTK_WINDOW(glb_win_app), TRUE);
+
+	if(!glb_fpm_ini->tr_always_visible)
+	    usleep(50000);
+
+	g_timeout_add(50, fpm_hide_win_app, NULL);
+
+	gtk_window_get_position(GTK_WINDOW(glb_win_app), &x, &y);
+	screen = gtk_window_get_screen(GTK_WINDOW(glb_win_app));
+
+	gtk_status_icon_get_geometry(glb_tray_icon, NULL, &bounds, NULL );
+	gdk_window = glb_win_app->window;
+	dpy = gdk_x11_drawable_get_xdisplay (gdk_window);
+
+	data[0] = bounds.x;
+	data[1] = bounds.y;
+	data[2] = bounds.width;
+	data[3] = bounds.height;
+
+	XChangeProperty (dpy,
+			GDK_WINDOW_XID (gdk_window),
+			gdk_x11_get_xatom_by_name_for_display (gdk_drawable_get_display (gdk_window),
+			"_NET_WM_ICON_GEOMETRY"),
+			XA_CARDINAL, 32, PropModeReplace,
+			(guchar*)&data, 4);
+
+	gtk_window_iconify(GTK_WINDOW(glb_win_app));
+
+	if (glb_fpm_ini->tr_auto_lock && !glb_locked) {
+	    g_timeout_add_seconds(glb_fpm_ini->tr_auto_lock_minutes*60, (GSourceFunc) fpm_lock, NULL);
+	}
+	glb_minimized = TRUE;
+    }
+}
+
+
+void fpm_tr_cleanup() {
+    if (glb_tray_icon) {
+	gtk_status_icon_set_visible (glb_tray_icon, FALSE);
+	glb_tray_icon = (g_object_unref (glb_tray_icon), NULL);
+    }
+}
+
+gboolean fpm_lock() {
+
+    glb_locked = TRUE;
+
+    if (!glb_minimized)
+	fpm_tr_toggle_win_app(TRUE);
+
+//    g_signal_handlers_disconnect_by_func(glb_tray_icon, G_CALLBACK (tray_icon_on_menu), NULL);
+
+    gtk_status_icon_set_tooltip(glb_tray_icon, _("Figaro's Password Manager 2 - locked"));
+    fpm_crypt_init("");
+    fpm_clear_list();
+    fpm_ini_save();
+
+    return FALSE;
+}
+
+gboolean fpm_window_check() {
+    GList* list;
+    list = gtk_window_list_toplevels();
+    while (list) {
+	if ((gtk_window_get_title(list->data) != NULL) && (list->data != glb_win_app))
+	    return TRUE;
+	list = g_list_next(list);
+    }
+    return FALSE;
+}
