@@ -5,7 +5,7 @@
 static gint
 fpm_cmp_title(fpm_data *a, fpm_data *b)
 {
-  return(g_strcasecmp(a->title, b->title));
+  return(g_utf8_collate(a->title, b->title));
 }
 
 void
@@ -27,7 +27,8 @@ fpm_create_category_list(gint edit_flag)
   g_list_free(glb_cat_string_list);
   glb_cat_string_list = NULL;
 
-  cmp_func = (GCompareFunc) g_strcasecmp;
+//  cmp_func = (GCompareFunc) g_strcasecmp;
+  cmp_func = (GCompareFunc) g_utf8_collate;
 
   /* Step through pass list adding to cat list */
   list = g_list_first(glb_pass_list);
@@ -37,14 +38,13 @@ fpm_create_category_list(gint edit_flag)
     if (
     /*
         strcmp(data->category, "") &&
-	*/
+    */
         g_list_find_custom(glb_cat_string_list, data->category, cmp_func) ==NULL)
     {
       g_assert(data->category !=NULL);
       glb_cat_string_list=g_list_insert_sorted(glb_cat_string_list,
       	g_strdup(data->category), cmp_func);
     }
-    
     list=g_list_next(list);
   }
 
@@ -53,10 +53,10 @@ fpm_create_category_list(gint edit_flag)
   if(edit_flag)
   {
     /* Add first item entry to beginning of list */
-    glb_cat_string_list=g_list_prepend(glb_cat_string_list, g_strdup(""));
+    glb_cat_string_list = g_list_prepend(glb_cat_string_list, g_strdup(""));
   }
   else
-  {
+  { 
     /* Add all category and default category */
     glb_cat_string_list=g_list_prepend(glb_cat_string_list, g_strdup(FPM_DEFAULT_CAT_MSG));
     glb_cat_string_list=g_list_prepend(glb_cat_string_list, g_strdup(FPM_ALL_CAT_MSG));
@@ -117,21 +117,24 @@ fpm_clist_init()
   cellRenderer = gtk_cell_renderer_text_new();
   treeViewColumn = gtk_tree_view_column_new_with_attributes (_("Title"), cellRenderer, "text", 0, NULL);
   gtk_tree_view_append_column (GTK_TREE_VIEW(glb_clist_main),treeViewColumn);
-  gtk_tree_view_column_set_resizable(treeViewColumn, 1);
+  gtk_tree_view_column_set_resizable (treeViewColumn, 1);
   gtk_tree_view_column_set_fixed_width (treeViewColumn, glb_fpm_ini->columns_width[0]);
-  gtk_tree_view_column_set_sort_column_id(treeViewColumn,1);
   gtk_tree_view_column_set_sizing (treeViewColumn, GTK_TREE_VIEW_COLUMN_FIXED);
+  gtk_tree_view_column_set_sort_column_id (treeViewColumn, 0);
    
   treeViewColumn = gtk_tree_view_column_new_with_attributes (_("URL"),cellRenderer,"text",1,NULL);
   gtk_tree_view_append_column (GTK_TREE_VIEW(glb_clist_main),treeViewColumn);
-  gtk_tree_view_column_set_resizable(treeViewColumn, 2);
+  gtk_tree_view_column_set_resizable (treeViewColumn, 2);
   gtk_tree_view_column_set_fixed_width (treeViewColumn, glb_fpm_ini->columns_width[1]);
   gtk_tree_view_column_set_sizing (treeViewColumn, GTK_TREE_VIEW_COLUMN_FIXED);
+  gtk_tree_view_column_set_sort_column_id (treeViewColumn, 1);
 
   treeViewColumn = gtk_tree_view_column_new_with_attributes (_("User"),cellRenderer,"text",2,NULL);
   gtk_tree_view_append_column (GTK_TREE_VIEW(glb_clist_main),treeViewColumn);
-  gtk_tree_view_column_set_resizable(treeViewColumn, 3);
+  gtk_tree_view_column_set_resizable (treeViewColumn, 3);
   gtk_tree_view_column_set_sizing (treeViewColumn, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+  gtk_tree_view_column_set_sort_column_id (treeViewColumn, 2);
+
 }
 
 
